#! /usr/bin/perl -w
#
# $Id: check_graph_load_average.pl,v 1.3 2005/08/11 11:38:41 wistof Exp $
#
# Oreon's plugins are developped with GPL Licence :
# http://www.fsf.org/licenses/gpl.txt
# Developped by : Jean Baptiste Gouret - Julien Mathis - Mathieu Mettre - Romain Le Merlus - Yohann Lecarpentier
# Under control of Flavien Astraud, Jerome Landrieu for Epitech.
# Oreon's plugins are developped in partnership with Linagora company.
#
#
# The Software is provided to you AS IS and WITH ALL FAULTS.
# OREON makes no representation and gives no warranty whatsoever,
# whether express or implied, and without limitation, with regard to the quality,
# safety, contents, performance, merchantability, non-infringement or suitability for
# any particular or intended purpose of the Software found on the OREON web site.
# In no event will OREON be liable for any direct, indirect, punitive, special,
# incidental or consequential damages however they may arise and even if OREON has
# been previously advised of the possibility of such damages.

##
## Plugin init
##
use strict;
use Net::SNMP qw(:snmp);
use FindBin qw($Bin);
use lib "$Bin";
use oreon;
use vars qw($VERSION %oreon);
use vars qw(%oreon);
BEGIN
{
    %oreon=oreon::get_parameters;
}

use lib $oreon{NAGIOS_LIBEXEC} ;
my $pathtorrdbase = $oreon{DIR_OREON}."rrd/";

use utils qw($TIMEOUT %ERRORS &print_revision &support);
use vars qw($PROGNAME);
use Getopt::Long;
use vars qw($opt_V $opt_h $opt_g $opt_v $opt_C $opt_H $opt_D $opt_S);

##
## Plugin var init
##
my($return_code);

$PROGNAME = "check_graph_load_average";
sub print_help ();
sub print_usage ();

$ENV{'PATH'}='';
$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';

Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"         => \$opt_h,
     "V"   => \$opt_V, "version"      => \$opt_V,
     "g"   => \$opt_g, "rrdgraph"     => \$opt_g,
     "D=s" => \$opt_D, "directory=s"  => \$opt_D,
     "v=s" => \$opt_v, "snmp=s"       => \$opt_v,
     "C=s" => \$opt_C, "community=s"  => \$opt_C,
     "S=s" => \$opt_S, "ServiceId=s"  => \$opt_S,
     "H=s" => \$opt_H, "hostname=s"   => \$opt_H);

if ($opt_V) {
    print_revision($PROGNAME,'$Revision: 1.3 $');
    exit $ERRORS{'OK'};
}

if ($opt_h) {
    print_help();
    exit $ERRORS{'OK'};
}

$opt_H = shift unless ($opt_H);
(print_usage() && exit $ERRORS{'OK'}) unless ($opt_H);


($opt_v) || ($opt_v = shift) || ($opt_v = "1");
my $snmp = $1 if ($opt_v =~ /(\d)/);

($opt_S) || ($opt_S = shift) || ($opt_S = 1);
my $ServiceId;
if ($opt_S =~ /([0-9]+)/){
    $ServiceId = $1;
}
else{
    print "Unknown -S Service ID expected... or it doesn't exist, try another id - number\n";
    exit $ERRORS{'UNKNOWN'};
}

($opt_C) || ($opt_C = shift) || ($opt_C = "public");

($opt_D) || ($opt_D = shift) || ($opt_D = $pathtorrdbase);
$pathtorrdbase = $1 if ($opt_D =~ /([-.\/\_\ A-Za-z0-9]+)/);

use lib qw(/usr/local/rrdtool/lib/perl ../lib/perl);
my $rrd = $pathtorrdbase.$ServiceId.".rrd";

use RRDs;
my $start=time;
my $name = $0;
$name =~ s/\.pl.*//g;
my $ERROR = RRDs::error;

##
## RRDTools create rrd
##
if ($opt_g) {
    if (-e $rrd) {;}
    else
    {
        RRDs::create ($rrd, "--start",$start-1, "--step",300,
            "DS:a:GAUGE:600:U:U",
            "DS:b:GAUGE:600:U:U",
            "DS:c:GAUGE:600:U:U",
            "RRA:AVERAGE:0.5:1:8440",
            "RRA:MIN:0.5:12:8440",
            "RRA:MAX:0.5:12:8440",
            );
        $ERROR = RRDs::error;
        if ($ERROR) {
            print "unable to create $rrd: $ERROR\n" ;
            exit $ERRORS{'UNKNOWN'};
        }
    }
}

##
## Plugin snmp requests
##
$return_code = 0;

my $OID_CPULOAD_1 =".1.3.6.1.4.1.2021.10.1.3.1";
my $OID_CPULOAD_5 =".1.3.6.1.4.1.2021.10.1.3.2";
my $OID_CPULOAD_15 =".1.3.6.1.4.1.2021.10.1.3.3";

my ( $session, $error ) = Net::SNMP->session(-hostname  => $opt_H,-community => $opt_C, -version  => $snmp);
if ( !defined($session) ) {
    print("UNKNOWN: $error");
    exit $ERRORS{'UNKNOWN'};
}

my $result = $session->get_request(
                                -varbindlist => [$OID_CPULOAD_1, $OID_CPULOAD_5, $OID_CPULOAD_15 ]
                                   );
if (!defined($result)) {
    printf("UNKNOWN: %s.\n", $session->error);
    $session->close;
    exit $ERRORS{'UNKNOWN'};
}

my $un =  $result->{$OID_CPULOAD_1};
my $cinq  =  $result->{$OID_CPULOAD_5};
my $quinze  =  $result->{$OID_CPULOAD_15};

##
## RRDtools update
##
if ($opt_g && ( $return_code == 0) ) {
    $start=time;
    RRDs::update $rrd, "$start:".$un.":".$cinq.":".$quinze;
    $ERROR = RRDs::error;
    if ($ERROR) {
        print "unable to update $rrd: $ERROR\n" ;
        exit $ERRORS{'UNKNOWN'};
    }
}

##
## Plugin return code
##

if ($return_code == 0){
    print "OK - load average: $un, $cinq, $quinze\n";
    exit $ERRORS{'OK'};
}
else{
        print "Load Average CRITICAL\n";
        exit $ERRORS{'CRITICAL'};
}

sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -H (--hostname)   Hostname to query - (required)\n";
    print "   -C (--community)  SNMP read community (defaults to public,\n";
    print "                     used with SNMP v1 and v2c\n";
    print "   -v (--snmp_version)  1 for SNMP v1 (default)\n";
    print "                        2 for SNMP v2c\n";
    print "   -g (--rrdgraph)   create   rrd base and add datas into this one\n";
    print "   -D (--directory)  Path to rrdatabase (or create the .rrd in this directory)\n";
    print "                     by default: ".$pathtorrdbase."\n";
    print "                     (The path is valid with spaces '/my\ path/...')\n";
    print "   -S (--ServiceId)  Oreon Service Id\n";
    print "   -V (--version)    Plugin version\n";
    print "   -h (--help)       usage help\n";

}

sub print_help () {
    print "Copyright (c) 2004 OREON\n";
    print "Bugs to http://www.oreon.org/\n";
    print "\n";
    print_usage();
    print "\n";
}
