#! /usr/bin/perl -w
#
# $Id: check_graph_nt.pl,v 1.5 2005/08/23 22:44:23 wistof Exp $
#
# Oreon's plugins are developped with GPL Licence :
# http://www.fsf.org/licenses/gpl.txt
# Developped by : Jean Baptiste Gouret - Julien Mathis - Mathieu Mettre - Romain Le Merlus - Yohann Lecarpentier
# Under control of Flavien Astraud, Jerome Landrieu for Epitech.
# Oreon's plugins are developped in partnership with Linagora company.
#
#
# The Software is provided to you AS IS and WITH ALL FAULTS.
# OREON makes no representation and gives no warranty whatsoever,
# whether express or implied, and without limitation, with regard to the quality,
# safety, contents, performance, merchantability, non-infringement or suitability for
# any particular or intended purpose of the Software found on the OREON web site.
# In no event will OREON be liable for any direct, indirect, punitive, special,
# incidental or consequential damages however they may arise and even if OREON has
# been previously advised of the possibility of such damages.

##
## Plugin init
##
use strict;
use FindBin qw($Bin);
use lib "$Bin";
use oreon;
use vars qw($VERSION %oreon);
use vars qw(%oreon);
BEGIN
{
    %oreon=oreon::get_parameters;
}

use lib $oreon{NAGIOS_LIBEXEC} ;
my $pathtorrdbase = $oreon{DIR_OREON}."rrd/";
my $pathtolibexecnt = $oreon{NAGIOS_LIBEXEC}."check_nt";

use utils qw($TIMEOUT %ERRORS &print_revision &support);
use vars qw($PROGNAME);
use Getopt::Long;
use vars qw($opt_H $opt_p $opt_s $opt_v $opt_V $opt_h $opt_w $opt_c $opt_S $opt_g $opt_t $opt_l $opt_d $opt_D);

##
## Plugin var init
##
my($op_v, $op_d, $op_s, $op_t, $op_l, $port, @test, @test2, @test3, @test4, @test5, $warning, $critical, @w, @c, $uptime);
my($warning2, $critical2, $warning3, $critical3, $warning4, $critical4);
$PROGNAME = "check_graph_nt";
sub print_help ();
sub print_usage ();

$ENV{'PATH'}='';
$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';

Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"         => \$opt_h,
     "p=s" => \$opt_p, "port=s"       => \$opt_p,
     "V"   => \$opt_V, "version"      => \$opt_V,
     "g"   => \$opt_g, "rrdgraph"     => \$opt_g,
     "s=s" => \$opt_s, "password=s"   => \$opt_s,
     "d=s" => \$opt_d, "showall=s"    => \$opt_d,
     "v=s" => \$opt_v, "variable=s"   => \$opt_v,
     "D=s" => \$opt_D, "directory=s"  => \$opt_D,
     "t=s" => \$opt_t, "timeout=s"    => \$opt_t,
     "l:s" => \$opt_l, "parameter:s"  => \$opt_l,
     "w=s" => \$opt_w, "warning=s"    => \$opt_w,
     "c=s" => \$opt_c, "critical=s"   => \$opt_c,
     "S=s" => \$opt_S, "ServiceId=s"  => \$opt_S,
     "H=s" => \$opt_H, "hostname=s"   => \$opt_H);

if ($opt_h) {
    print_help();
    exit $ERRORS{'OK'};
}

if ($opt_V) {
    $_ = `$pathtolibexecnt -V`;
    print "$_";
    exit $ERRORS{'OK'};
}

if ($opt_p) {
  if ($opt_p =~ /([0-9]+)/){
    $port = $1;
  }
  else{
    print "Unknown -p number expected... or it doesn't exist, try another port - number\n";
    exit $ERRORS{'UNKNOWN'};
  }
}

$opt_H = shift unless ($opt_H);
(print_usage() && exit $ERRORS{'OK'}) unless ($opt_H);


if ($opt_c) {
  ($opt_c) || ($opt_c = shift);
  $critical = $1 if ($opt_c =~ /([0-9]+)/);
}

if ($opt_w) {
  ($opt_w) || ($opt_w = shift);
  $warning = $1 if ($opt_w =~ /([0-9]+)/);
}

if (($critical && $warning) && ($critical <= $warning)) {
    print "(--crit) must be superior to (--warn)";
    print_usage();
    exit $ERRORS{'OK'};
}

if ($opt_l) {
  ($opt_l) || ($opt_l = shift);
  $op_l = $1 if ($opt_l =~ /([-.,A-Za-z0-9]+)/);
}

if ($opt_t) {
  ($opt_t) || ($opt_t = shift);
  $op_t = $1 if ($opt_t =~ /([-.,A-Za-z0-9]+)/);
}

if ($opt_s) {
  ($opt_s) || ($opt_s = shift);
  $op_s = $1 if ($opt_s =~ /([-.,A-Za-z0-9]+)/);
}

if ($opt_d) {
  ($opt_d) || ($opt_d = shift);
  $op_d = $1 if ($opt_d =~ /([-.,A-Za-z0-9]+)/);
}

if ($opt_v) {
  ($opt_v) || ($opt_v = shift);
  $op_v = $1 if ($opt_v =~ /([-.,A-Za-z0-9]+)/);
}

($opt_D) || ($opt_D = shift) || ($opt_D = $pathtorrdbase);
$pathtorrdbase = $1 if ($opt_D =~ /([-.\/\_\ A-Za-z0-9]+)/);

($opt_S) || ($opt_S = shift) || ($opt_S = 1);
my $ServiceId;
if ($opt_S =~ /([0-9]+)/){
    $ServiceId = $1;
}
else{
    print "Unknown -S Service ID expected... or it doesn't exist, try another id - number\n";
    exit $ERRORS{'UNKNOWN'};
}

##
## RRDTool var init
##
use lib qw(/usr/local/rrdtool/lib/perl ../lib/perl);
my $rrd = $pathtorrdbase.$ServiceId.".rrd";
use RRDs;
my $start=time;
my $name = $0;
$name =~ s/\.pl.*//g;
my $ERROR = RRDs::error;
my $return_code;
##
## Plugin requests
##
if ($op_v) {
    if ($op_v) {$op_v = "-v ".$op_v;}
    if ($port) {$port = "-p ".$port;} else { $port = " ";}
    if ($warning) {$warning = "-w ".$warning;} else { $warning = " ";}
    if ($critical) {$critical = "-c ".$critical;} else { $critical = " ";}
    if ($op_l) {$op_l = "-l ".$op_l;} else { $op_l = " ";}
    if ($op_t) {$op_t = "-t ".$op_t;} else { $op_t = " ";}
    if ($op_s) {$op_s = "-s ".$op_s;} else { $op_s = " ";}
    if ($op_d) {$op_d = "-d ".$op_d;} else { $op_d = " ";}
    $_ = `$pathtolibexecnt -H $opt_H $op_v $port $warning $critical $op_l $op_t $op_s $op_d 2>/dev/null`;
    my $return = $_;
    $return =~ s/\\//g;
    $return_code = $?;
    $return_code = $return_code / 256;
   # if ( $return_code != 0) {
   #     print "$return";
   #     exit $ERRORS{'CRITICAL'};
   # }
    ##
    ## CLIENTVERSION
    ##
    if ($op_v =~ /CLIENTVERSION/){
        print "CLIENTVERSION impossible to Graph!\n";
        exit $ERRORS{'UNKNOWN'};
    }
    ##
    ## CPULOAD
    ##
    if (($op_v =~ /CPULOAD/) && ($op_l =~ /([-.,A-Za-z0-9]+)/)){
        @test = split(/ /,$_);

        @test2 = split(/\%/, $test[2]);
        @c = split(/,/, $op_l);
        $warning = $c[1];
        $critical = $c[2];

        if (($c[4]) && ($c[4] =~ /([-.,A-Za-z0-9]+)/)) {
            @test3 = split(/\%/, $test[6]);
            $warning2 = $c[4];
            $critical2 = $c[5];
        }

        if (($c[7]) && ($c[7] =~ /([-.,A-Za-z0-9]+)/)) {
            @test4 = split(/\%/, $test[10]);
            $warning3 = $c[7];
            $critical3 = $c[8];
        }

        if (($c[10]) && ($c[10] =~ /([-.,A-Za-z0-9]+)/)) {
            @test5 = split(/\%/, $test[14]);
            $warning4 = $c[10];
            $critical4 = $c[11];
        }

        ##
        ## RRDTool create rrd
        ##
        if (($opt_g) && ($c[10])) {
            unless (-e $rrd) {
                RRDs::create ($rrd, "--start",$start-1, "--step",300,
                    "DS:a:GAUGE:600:U:U",
                    "DS:b:GAUGE:600:U:U",
                    "DS:c:GAUGE:600:U:U",
                    "DS:d:GAUGE:600:U:U",
                    "RRA:AVERAGE:0.5:1:8440",
                    "RRA:MIN:0.5:12:8440",
                    "RRA:MAX:0.5:12:8440",
                    );
                $ERROR = RRDs::error;
                if ($ERROR) {
                    print "unable to create $rrd: $ERROR\n" ;
                    exit $ERRORS{'UNKNOWN'};
                }
            }
        }
        if (($opt_g) && ($c[7])) {
            unless (-e $rrd) {
                RRDs::create ($rrd, "--start",$start-1, "--step",300,
                    "DS:a:GAUGE:600:U:U",
                    "DS:b:GAUGE:600:U:U",
                    "DS:c:GAUGE:600:U:U",
                    "RRA:AVERAGE:0.5:1:8440",
                    "RRA:MIN:0.5:12:8440",
                    "RRA:MAX:0.5:12:8440",
                    );
                $ERROR = RRDs::error;
                if ($ERROR) {
                    print "unable to create $rrd: $ERROR\n" ;
                    exit $ERRORS{'UNKNOWN'};
                }
            }
        }

        if (($opt_g) && ($c[4])) {
            unless (-e $rrd) {
                RRDs::create ($rrd, "--start",$start-1, "--step",300,
                    "DS:a:GAUGE:600:U:U",
                    "DS:b:GAUGE:600:U:U",
                    "RRA:AVERAGE:0.5:1:8440",
                    "RRA:MIN:0.5:12:8440",
                    "RRA:MAX:0.5:12:8440",
                    );
                $ERROR = RRDs::error;
                if ($ERROR) {
                    print "unable to create $rrd: $ERROR\n" ;
                    exit $ERRORS{'UNKNOWN'};
                }
            }
        }

        if (($opt_g) && ($c[1])) {
            unless (-e $rrd) {
                RRDs::create ($rrd, "--start",$start-1, "--step",300,
                    "DS:a:GAUGE:600:U:U",
                    "RRA:AVERAGE:0.5:1:8440",
                    "RRA:MIN:0.5:12:8440",
                    "RRA:MAX:0.5:12:8440",
                );
                $ERROR = RRDs::error;
                if ($ERROR) {
                    print "unable to create $rrd: $ERROR\n" ;
                    exit $ERRORS{'UNKNOWN'};
                }
            }
        }

        if (($opt_g) && ($c[10]) && (defined $test2[0]) && (defined $test3[0] ) && ( defined $test4[0]) && ( defined $test5[0]) ) {
            $start=time;
            RRDs::update $rrd, "$start:".$test2[0].":".$test3[0].":".$test4[0].":".$test5[0];
            $ERROR = RRDs::error;
            if ($ERROR) {
                print "unable to update $rrd: $ERROR\n" ;
                exit $ERRORS{'UNKNOWN'};
            }
        } elsif (($opt_g) && ($c[7]) && (defined $test2[0]) && (defined $test3[0] ) && ( defined $test4[0]) ) {
            $start=time;
            RRDs::update $rrd, "$start:".$test2[0].":".$test3[0].":".$test4[0];
            $ERROR = RRDs::error;
            if ($ERROR) {
                print "unable to update $rrd: $ERROR\n" ;
                exit $ERRORS{'UNKNOWN'};
            }
        } elsif (($opt_g) && ($c[4]) && (defined $test2[0]) && (defined $test3[0] ) ) {
            $start=time;
            RRDs::update $rrd, "$start:".$test2[0].":".$test3[0];
            $ERROR = RRDs::error;
            if ($ERROR) {
                print "unable to update $rrd: $ERROR\n" ;
                exit $ERRORS{'UNKNOWN'};
            }
        } elsif (($opt_g) && ($c[1]) && (defined $test2[0])) {
            $start=time;
            RRDs::update $rrd, "$start:".$test2[0];
            $ERROR = RRDs::error;
            if ($ERROR) {
                print "unable to update $rrd: $ERROR\n" ;
                exit $ERRORS{'UNKNOWN'};
            }
        }

        ## Print Plugins Output
        print $return;
		exit $return_code;
        #if ( $return_code > 0 ) {
        #    exit $return_code;
        #} elsif (($critical) && ($test2[0] >= $critical)){
        #    exit $ERRORS{'CRITICAL'};
        #} elsif (($warning) && ($test2[0] >= $warning)){
        #    exit $ERRORS{'WARNING'};
        #} elsif (($critical2) && ($test3[0] >= $critical2)){
        #    exit $ERRORS{'CRITICAL'};
        #} elsif (($warning2) && ($test3[0] >= $warning2)){
        #    exit $ERRORS{'WARNING'};
        #} elsif (($critical3) && ($test4[0] >= $critical3)){
        #    exit $ERRORS{'CRITICAL'};
        #} elsif (($warning3) && ($test4[0] >= $warning3)){
        #    exit $ERRORS{'WARNING'};
        #} elsif (($critical4) && ($test5[0] >= $critical4)){
        #    exit $ERRORS{'CRITICAL'};
        #} elsif (($warning4) && ($test5[0] >= $warning4)){
        #    exit $ERRORS{'WARNING'};
        #} else {
        #    exit $ERRORS{'OK'};
        #}

       }
    ##
    ## UPTIME
    ##
    if ($op_v =~ /UPTIME/){
        if ($_ =~ /.*[-:]+\s(\d+)\s.*$/ ) {
            $uptime = $1;
        } else {
            print "unable to parse check_nt output: $_\n" ;
            exit $ERRORS{'UNKNOWN'};
        }

        if ($opt_g) {
           if (-e $rrd) {
            $start=time;
            RRDs::update $rrd, "$start:".$uptime;
            $ERROR = RRDs::error;
            if ($ERROR) {
                print "unable to create $rrd: $ERROR\n" ;
                exit $ERRORS{'UNKNOWN'};
            }
        } else
            {
               RRDs::create ($rrd, "--start",$start-1, "--step",300,
               "DS:a:GAUGE:600:U:U",
               "RRA:AVERAGE:0.5:1:8440",
               "RRA:MIN:0.5:12:8440",
               "RRA:MAX:0.5:12:8440",
               );
               $ERROR = RRDs::error;
                if ($ERROR) {
                    print "unable to create $rrd: $ERROR\n" ;
                    exit $ERRORS{'UNKNOWN'};
                }
           }
        }

        print $_ ;
        exit $ERRORS{'OK'};

    }
    ##
    ## USEDDISKSPACE
    ##
    if (($op_v =~ /USEDDISKSPACE/) && ($op_l =~ /([-.,A-Za-z0-9]+)/)){
        @test = split(/ /,$_);
        @test2 = split(/\(/, $test[9]);
        @test3 = split(/\%/, $test2[1]);
        @c = split(/ /, $critical);
        $critical = $c[1];
        @w = split(/ /, $warning);
        $warning = $w[1];

        if ($opt_g) {
            unless (-e $rrd) {
                RRDs::create ($rrd, "--start",$start-1, "--step",300,
                "DS:a:GAUGE:600:U:U",
                "DS:b:GAUGE:600:U:U",
                "DS:c:GAUGE:600:U:U",
                "RRA:AVERAGE:0.5:1:8440",
                "RRA:MIN:0.5:12:8440",
                "RRA:MAX:0.5:12:8440",
                );
                $ERROR = RRDs::error;
                if ($ERROR) {
                    print "unable to create $rrd: $ERROR\n" ;
                    exit $ERRORS{'UNKNOWN'};
                }
            }
        }

        if ($opt_g && (defined $test[3]) && (defined $test[7] )&& (defined $test[12]) ) {
            $start=time;
            # Replace , by . to convert in real float number (for rrdtool)
            $test[3] =~ s/,/\./ ;
            $test[7] =~ s/,/\./ ;
            $test[12] =~ s/,/\./ ;
            RRDs::update $rrd, "$start:".$test[3].":".$test[7].":".$test[12];
            $ERROR = RRDs::error;
            if ($ERROR) {
                print "unable to update $rrd: $ERROR\n" ;
                exit $ERRORS{'UNKNOWN'};
            }
        }

        ## Print Plugins Output
        print $return;
		exit $return_code;
        #if ( $return_code > 0 ) {  exit $return_code; }
        #    if (($critical) && ($test3[0] >= $critical)){
        #        exit $ERRORS{'CRITICAL'};
        #    } elsif (($warning) && ($test3[0] >= $warning)){
        #        exit $ERRORS{'WARNING'};
        #    } else {
        #    exit $ERRORS{'OK'};
        #    }
    }
    ##
    ## MEMUSE
    ##
    if ($op_v =~ /MEMUSE/){
        @test = split(/ /,$_);
        @test4 = split(/:/, $test[2]);
        @test2 = split(/\(/, $test[8]);
        @test3 = split(/\%/, $test2[1]);
        @c = split(/ /, $critical);
        $critical = $c[1];
        @w = split(/ /, $warning);
        $warning = $w[1];

         if ($opt_g) {
            unless (-e $rrd) {
                RRDs::create ($rrd, "--start",$start-1, "--step",300,
                "DS:a:GAUGE:600:U:U",
                "DS:b:GAUGE:600:U:U",
                "DS:c:GAUGE:600:U:U",
                "RRA:AVERAGE:0.5:1:8440",
                "RRA:MIN:0.5:12:8440",
                "RRA:MAX:0.5:12:8440",
                );
                $ERROR = RRDs::error;
                if ($ERROR) {
                    print "unable to create $rrd: $ERROR\n" ;
                    exit $ERRORS{'UNKNOWN'};
                }
            }
        }

        if ($opt_g && (defined $test4[1]) && (defined $test[6] ) && (defined $test[11]) ) {
            $start=time;
            # Replace , by . to convert in real float number (for rrdtool)
            $test4[1] =~ s/,/\./ ;
            $test[6] =~ s/,/\./ ;
            $test[11] =~ s/,/\./ ;
            RRDs::update $rrd, "$start:".$test4[1].":".$test[6].":".$test[11];
            $ERROR = RRDs::error;
            if ($ERROR) {
                print "unable to update $rrd: $ERROR\n" ;
                exit $ERRORS{'UNKNOWN'};
            }
        }

        ## Print Plugins Output
        print $return;
        exit $return_code;
        #if ( $return_code > 0 ){
        #    exit $return_code;
        #}

        #if (($critical) && ($test3[0] >= $critical)){
        #    exit $ERRORS{'CRITICAL'};
        #} elsif (($warning) && ($test3[0] >= $warning)){
        #    exit $ERRORS{'WARNING'};
        #} else {
        #    exit $ERRORS{'OK'};
        #}
    }
    ##
    ## SERVICESTATE
    ##
    if ($op_v =~ /SERVICESTATE/){
        print "SERVICESTATE not graphed\n";
        exit $ERRORS{'UNKNOWN'};
    }
    ##
    ## PROCSTATE
    ##
    if ($op_v =~ /PROCSTATE/){
        print "PROCSTATE not graphed\n";
        exit $ERRORS{'UNKNOWN'};
    }
    ##
    ## COUNTER
    ##
    if ($op_v =~ /COUNTER/){
        print "COUNTER not graphed\n";
        exit $ERRORS{'UNKNOWN'};
    }
} else {
    print "Could not parse arguments\n";
    exit $ERRORS{'UNKNOWN'};
}

##
## Plugin return code
##

sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   Usage: check_graph_nt -H host -v variable [-p port] [-w warning] [-c critical] [-l params] [-d SHOWALL] [-t timeout] -g\n";
    print "   Options:\n";
    print "   -H, --hostname=HOST\n";
    print "      Name of the host to check\n";
    print "   -p, --port=INTEGER\n";
    print "      Optional port number (default: 1248)\n";
    print "   -s <password>\n";
    print "      Password needed for the request\n";
    print "   -v, --variable=STRING\n";
    print "      Variable to check.  Valid variables are:\n";
    print "         CLIENTVERSION = Not Graphed. Get the NSClient version\n";
    print "         CPULOAD = Average CPU load on last x minutes. Request a -l parameter with the following syntax:\n";
    print "           -l <minutes range>,<warning threshold>,<critical threshold>. <minute range> should be less than 24*60.\n";
    print "          Thresholds are percentage and up to 10 requests can be done in one shot. ie: -l 60,90,95,120,90,95\n";
    print "          and 4 requests can be graphed.\n";
    print "         UPTIME = Only Days are graphed. Get the uptime of the machine. No specific parameters. No warning or critical threshold.\n";
    print "         USEDDISKSPACE = Size and percentage of disk use. Request a -l parameter containing the drive letter only.\n";
    print "                         Warning and critical thresholds can be specified with -w and -c.\n";
    print "         MEMUSE = Memory use. Warning and critical thresholds can be specified with -w and -c.\n";
    print "         SERVICESTATE = Not Graphed. Check the state of one or several services. Request a -l parameters with the following syntax:\n";
    print "           -l <service1>,<service2>,<service3>,... You can specify -d SHOWALL in case you want to see working services\n";
    print "           in the returned string.\n";
    print "         PROCSTATE = Not Graphed. Check if one or several process are running. Same syntax as SERVICESTATE.\n";
    print "         COUNTER = Not Graphed. Check any performance counter of Windows NT/2000. Request a -l parameters with the following syntax:\n";
    print "           -l \"<performance object>counter\",\"<description>\"  The <description> parameter is optional and\n";
    print "           is given to a printf output command which require a float parameters. Some examples:\n";
    print "             \"Paging file usage is %.2f %%\" or \"%.f %% paging file used.\"\n";
    print "    -w, --warning=INTEGER\n";
    print "      Threshold which will result in a warning status\n";
    print "    -c, --critical=INTEGER\n";
    print "      Threshold which will result in a critical status\n";
    print "    -t, --timeout=INTEGER\n";
    print "      Seconds before connection attempt times out (default: 10)\n";
    print "   -h, --help\n";
    print "      Print this help screen\n";
    print "   -V, --version\n";
    print "      Print version information\n";
    print "   -g (--rrdgraph)   create   rrd base and add datas into this one\n";
    print "   -D (--directory)  Path to rrdatabase (or create the .rrd in this directory)\n";
    print "                     by default: ".$pathtorrdbase."\n";
    print "                     (The path is valid with spaces '/my\ path/...')\n";
    print "   -S (--ServiceId)  Oreon Service Id\n";
    print "   -V (--version)    Plugin version\n";
    print "   -h (--help)       usage help\n";

}

sub print_help () {
    print "Copyright (c) 2004 OREON\n";
    print "Bugs to http://www.oreon.org/\n";
    print "\n";
    print_usage();
    print "\n";
}
