#! /usr/bin/perl -w
#
# $Id: check_graph_process.pl,v 1.4 2005/08/22 18:41:27 wistof Exp $
#
# Oreon's plugins are developped with GPL Licence :
# http://www.fsf.org/licenses/gpl.txt
# Developped by : Jean Baptiste Gouret - Julien Mathis - Mathieu Mettre - Romain Le Merlus - Yohann Lecarpentier
# Under control of Flavien Astraud, Jerome Landrieu for Epitech.
# Oreon's plugins are developped in partnership with Linagora company.
#
#
# The Software is provided to you AS IS and WITH ALL FAULTS.
# OREON makes no representation and gives no warranty whatsoever,
# whether express or implied, and without limitation, with regard to the quality,
# safety, contents, performance, merchantability, non-infringement or suitability for
# any particular or intended purpose of the Software found on the OREON web site.
# In no event will OREON be liable for any direct, indirect, punitive, special,
# incidental or consequential damages however they may arise and even if OREON has
# been previously advised of the possibility of such damages.

##
## Plugin init
##
use strict;
use Net::SNMP qw(:snmp oid_lex_sort);
use FindBin qw($Bin);
use lib "$Bin";
use oreon;
use vars qw($VERSION %oreon);
use vars qw(%oreon);
BEGIN
{
    %oreon=oreon::get_parameters;
}

use lib $oreon{NAGIOS_LIBEXEC} ;
my $pathtorrdbase = $oreon{DIR_OREON}."rrd/";

use utils qw($TIMEOUT %ERRORS &print_revision &support);
use vars qw($PROGNAME);
use Getopt::Long;
use vars qw($opt_V $opt_h $opt_g $opt_v $opt_C $opt_p $opt_H $opt_D $opt_n $opt_S $result @result %process_list %STATUS);

##
## Plugin var init
##
my($proc, $proc_run);

$PROGNAME = "check_graph_process";
sub print_help ();
sub print_usage ();

$ENV{'PATH'}='';
$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';

%STATUS=(1=>'running',2=>'runnable',3=>'notRunnable',4=>'invalid');


Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"         => \$opt_h,
     "V"   => \$opt_V, "version"      => \$opt_V,
     "g"   => \$opt_g, "rrdgraph"     => \$opt_g,
     "D=s" => \$opt_D, "directory=s"  => \$opt_D,
     "n"   => \$opt_n, "number"       => \$opt_n,
     "v=s" => \$opt_v, "snmp=s"       => \$opt_v,
     "C=s" => \$opt_C, "community=s"  => \$opt_C,
     "p=s" => \$opt_p, "process=s"    => \$opt_p,
     "S=s" => \$opt_S, "ServiceId=s"  => \$opt_S,
     "H=s" => \$opt_H, "hostname=s"   => \$opt_H);

if ($opt_V) {
    print_revision($PROGNAME,'$Revision: 1.4 $');
    exit $ERRORS{'OK'};
}

if ($opt_h) {
    print_help();
    exit $ERRORS{'OK'};
}

$opt_H = shift unless ($opt_H);
(print_usage() && exit $ERRORS{'OK'}) unless ($opt_H);

($opt_v) || ($opt_v = shift) || ($opt_v = "1");
my $snmp = $1 if ($opt_v =~ /(\d)/);

($opt_S) || ($opt_S = shift) || ($opt_S = 1);
my $ServiceId;
if ($opt_S =~ /([\d]+)/){
    $ServiceId = $1;
}
else{
    print "Unknown -S Service ID expected... or it doesn't exist, try another id - number\n";
    exit $ERRORS{'UNKNOWN'};
}

($opt_C) || ($opt_C = shift) || ($opt_C = "public");

($opt_p) || ($opt_p = shift) || ($opt_p = "smbd");
if (!$opt_p){print_usage();exit $ERRORS{'OK'};}
my $process = $1 if ($opt_p =~ /([-\.\w]+)/);

($opt_D) || ($opt_D = shift) || ($opt_D = $pathtorrdbase);
$pathtorrdbase = $1 if ($opt_D =~ /([-\.\/\ \w]+)/);

use lib qw(/usr/local/rrdtool/lib/perl ../lib/perl);
my $rrd = $pathtorrdbase.$ServiceId.".rrd";
use RRDs;
my $start=time;
my $name = $0;
$name =~ s/\.pl.*//g;
my $ERROR = RRDs::error;

##
## RRDTools create rrd
##
if ( $opt_g && $opt_n)   {
    if (-e $rrd) {;}
    else
    {
        RRDs::create ($rrd, "--start",$start-1, "--step",300,
            "DS:a:GAUGE:600:U:U",
            "RRA:AVERAGE:0.5:1:8440",
            "RRA:MIN:0.5:12:8440",
            "RRA:MAX:0.5:12:8440",
            );
        $ERROR = RRDs::error;
        if ($ERROR) {
            print "unable to create $rrd: $ERROR\n" ;
            exit $ERRORS{'UNKNOWN'};
        }
    }
}

##
## Plugin snmp requests
##
my $OID_SW_RunName =".1.3.6.1.2.1.25.4.2.1.2";
my $OID_SW_RunIndex =".1.3.6.1.2.1.25.4.2.1.1";
my $OID_SW_RunStatus =".1.3.6.1.2.1.25.4.2.1.7";

my ( $session, $error ) = Net::SNMP->session(-hostname  => $opt_H,-community => $opt_C, -version  => $snmp);
if ( !defined($session) ) {
    print("UNKNOWN: $error");
    exit $ERRORS{'UNKNOWN'};
}

 $result = $session->get_table(
        Baseoid => $OID_SW_RunName
    );
    if (!defined($result)) {
        printf("UNKNOWN: %s.\n", $session->error);
        $session->close;
        exit $ERRORS{'UNKNOWN'};
    }

     foreach my $key ( oid_lex_sort(keys %$result)) {
        my @oid_list = split (/\./,$key);
        $process_list{$$result{$key}} =  pop (@oid_list) ;
        $proc ++ ;
    }



if (!($opt_n))
{
    if ($process_list{$process}) {
        $result = $session->get_request(
                                        -varbindlist => [$OID_SW_RunStatus . "." . $process_list{$process} ]
                                           );
        if (!defined($result)) {
            printf("UNKNOWN: %s.\n", $session->error);
            $session->close;
            exit $ERRORS{'UNKNOWN'};
        }

    $proc_run =  $result->{$OID_SW_RunStatus . "." . $process_list{$process} };

    }
}

##
## RRDtools update
##
if ( $opt_g && $opt_n) {
    $start=time;
    my $totrrd;
    if ($opt_n){$totrrd = $proc;}
    else{
        if ( ($proc_run == "3") ||  ($proc_run == "4") ){$totrrd = 0;}
        else{$totrrd = 1;}
    }
    RRDs::update $rrd, "$start:".$totrrd;
    $ERROR = RRDs::error;
    if ($ERROR) {
        print "unable to update $rrd: $ERROR\n" ;
        exit $ERRORS{'UNKNOWN'};
    }
}

##
## Plugin return code
##
if ($opt_n){
    print "Processes OK - Number of current processes: $proc\n";
    exit $ERRORS{'OK'};
}
else
{
    if ($proc_run){
        print "Process OK - $process: $STATUS{$proc_run}\n";
        exit $ERRORS{'OK'};
    }
    else{
        print "Process CRITICAL - $process not in 'running' state\n";
        exit $ERRORS{'CRITICAL'};
    }
}

sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -H (--hostname)   Hostname to query - (required)\n";
    print "   -C (--community)  SNMP read community (defaults to public,\n";
    print "                     used with SNMP v1 and v2c\n";
    print "   -v (--snmp_version)  1 for SNMP v1 (default)\n";
    print "                        2 for SNMP v2c\n";
    print "   -g (--rrdgraph)   create   rrd base and add datas into this one\n";
    print "   -D (--directory)  Path to rrdatabase (or create the .rrd in this directory)\n";
    print "                     by default: ".$pathtorrdbase."\n";
    print "                     (The path is valid with spaces '/my\ path/...')\n";
    print "   -n (--number)     Return the number of current running processes. \n";
    print "   -p (--process)    Set the process name ex: by default smbd\n";
    print "   -S (--ServiceId)  Oreon Service Id\n";
    print "   -V (--version)    Plugin version\n";
    print "   -h (--help)       usage help\n";

}

sub print_help () {
    print "Copyright (c) 2004 OREON\n";
    print "Bugs to http://www.oreon.org/\n";
    print "\n";
    print_usage();
    print "\n";
}
