#! /usr/bin/perl -w
# $Id: check_graph_tcp.pl,v 1.4 2005/08/16 12:17:35 wistof Exp $
#
# This plugin is developped under GPL Licence:
# http://www.fsf.org/licenses/gpl.txt

# Developped by Linagora SA: http://www.linagora.com

# The Software is provided to you AS IS and WITH ALL FAULTS.
# LINAGORA makes no representation and gives no warranty whatsoever,
# whether express or implied, and without limitation, with regard to the quality,
# safety, contents, performance, merchantability, non-infringement or suitability for
# any particular or intended purpose of the Software found on the LINAGORA web site.
# In no event will LINAGORA be liable for any direct, indirect, punitive, special,
# incidental or consequential damages however they may arise and even if LINAGORA has
# been previously advised of the possibility of such damages.

# based on "graph plugins" developped by Oreon Team. See http://www.oreon.org.

use strict;
use Net::SNMP qw(:snmp);
use FindBin qw($Bin);
use lib "$Bin";
use oreon;
use vars qw($VERSION %oreon);
use vars qw(%oreon);
BEGIN
{
    %oreon=oreon::get_parameters;
}

use lib $oreon{NAGIOS_LIBEXEC} ;
my $pathtorrdbase = $oreon{DIR_OREON}."rrd/";
my $pathtolibexectcp = $oreon{NAGIOS_LIBEXEC}."check_tcp";


use utils qw($TIMEOUT %ERRORS &print_revision &support);

use Getopt::Long;
use vars qw($opt_V $opt_h $opt_p $opt_c $opt_w $opt_H $opt_S $opt_g $opt_D);
use RRDs;

sub print_help ();
sub print_usage ();


# get options
Getopt::Long::Configure('bundling');
GetOptions
    ("h|help"   		=> \$opt_h,
     "V|version"   		=> \$opt_V,
     "H|hostname=s" 	=> \$opt_H,
     "p|port=s" 		=> \$opt_p,
     "w|warning=s" 		=> \$opt_w,
     "c|critical=s" 	=> \$opt_c,
     "S|ServiceId=s" 	=> \$opt_S,
     "D|directory=s" 	=> \$opt_D,
     "g|rrdgraph"   	=> \$opt_g);

# version of plugin
if (defined($opt_V)) {
        print_revision($0,'$Revision: 1.4 $');
        exit $ERRORS{'OK'};
}

# help required
if (defined($opt_h)) {
        print_help();
        exit $ERRORS{'OK'};
}


$opt_H = shift unless ($opt_H);
(print_usage() && exit $ERRORS{'OK'}) unless ($opt_H);


($opt_S) || ($opt_S = shift) || ($opt_S = 1);
my $ServiceId;
if ($opt_S =~ /([0-9]+)/){
    $ServiceId = $1;
}
else{
    print "Unknown -S Service ID expected... or it doesn't exist, try another id - number\n";
    exit $ERRORS{'UNKNOWN'};
}
($opt_D) || ($opt_D = shift) || ($opt_D = $pathtorrdbase);
$pathtorrdbase = $1 if ($opt_D =~ /([-.\/\_\ A-Za-z0-9]+)/);

my $args_check_tcp = "-H $opt_H -p $opt_p";

if ( $opt_w ) {
    $args_check_tcp .= " -w $opt_w";
}

if ( $opt_c ) {
    $args_check_tcp .= " -c $opt_c";
}

my $rrd = $pathtorrdbase.$ServiceId.".rrd";
my $start=time;
my $ERROR = RRDs::error;

##
## RRDTools create rrd
##
if ($opt_g) {
        if (-e $rrd) {;}
        else
        {
                RRDs::create ($rrd, "--start",$start-1, "--step",300,
                        "DS:a:GAUGE:600:U:U",
                        "RRA:AVERAGE:0.5:1:8440",
                        "RRA:MIN:0.5:12:8440",
                        "RRA:MAX:0.5:12:8440",
                        );
                $ERROR = RRDs::error;
                if ($ERROR) {
                    print " unable to create `$rrd': $ERROR\n";
                    exit $ERRORS{'UNKNOWN'};
                }
        }
}


my $result = `$pathtolibexectcp $args_check_tcp`;
my $return_code = $? / 256;
$_ = $result;
m/(\d*\.\d*) second/;
my $time = $1;

#print "time: $time";
##
## RRDtools update
##
if ($opt_g && $time ) {
        $start=time;
        RRDs::update $rrd, "$start:".$time;
        $ERROR = RRDs::error;
        if ($ERROR) {
            print " unable to update `$rrd': $ERROR\n";
            exit $ERRORS{'UNKNOWN'};
        }
}



print "$result";
exit $return_code;



sub print_usage () {
    print "\nUsage:\n";
    print $0."\n";
    print "   -H (--hostname)   Hostname to query - (required)\n";
    print "   -p, --port=INTEGER Port number - (required)\n";
    print "   -w, --warning=DOUBLE Response time to result in warning status (seconds)\n";
    print "   -c, --critical=DOUBLE Response time to result in critical status (seconds)\n";
    print "   -V (--version)    Plugin version\n";
    print "   -h (--help)       usage help\n";

}

sub print_help () {
    print "Copyright (c) 2005 LINAGORA SA\n";
    print "Bugs to http://www.linagora.com/\n";
    print "\n";
    print_usage();
    print "\n";
}






